/*
* Copyright 2019 NXP
* SPDX-License-Identifier: BSD-3-Clause
*/

#include <peripherals_manual.h>
#include "fsl_common.h"
#include "button.h"
#include "MC_statemachine.h"
#include "freemaster.h"
#include "PMSM_SpeedVectorCtrl.h"
#include "mlib.h"

extern MCSTRUC_FOC_PMSM_OBS_DQ_T			gsMC_Drive;	
uint16_t w16ExeTime;
uint32_t u32Cnt;

extern BUTTON_CTRL_TYPE sSW;



__attribute__((section("RamFunction"))) void ADC_ETC_IRQ2_IRQHandler(void)
//__attribute__((section("RamFunction"))) void ADC_ETC_IRQ2_DriverIRQHandler(void)
//void ADC_ETC_IRQ2_IRQHandler(void)
{
	uint16_t u16ADC1_SEG0, u16ADC1_SEG1, u16ADC2_SEG0, u16ADC2_SEG1;

	TMR1->CHANNEL[0].CNTR = 0;

	// Clear flag
	ADC_ETC->DONE2_3_ERR_IRQ = ADC_ETC_DONE2_3_ERR_IRQ_TRIG4_DONE2_MASK;

	/*
	 *           TRIG0 ->  ADC1_SEG0(Ia), ADC1_SEG1(Udc)
	 *           TRIG4 ->  ADC2_SEG0(Ib), ADC2_SEG1(Ic)
	 *
	 *           ADC input voltage is scaled to 30/64 internally by HW. ADC reference is 1.8V, to make sure input 3.3 will be
	 *           corresponding to 1.8V, SW needs to add another scale.
	 *
	 *           C0 = 3.3*30/64
	 *           C1 = 1.8/C0 = 1.8*64/(3.3*30) = 1.636363 = 0.58182*2
	 *
	 *           (ADC_RSLT<<3)*C1 will be in range of 0~32767 when input voltage is 0~3.3V
	 *
	 * */
    /*========================= Read currents/DC bus voltage/IPM temperature begin =========================*/
	u16ADC1_SEG0 = (uint16_t)(ADC_ETC->TRIG[0].TRIGn_RESULT_1_0 & ADC_ETC_TRIGn_RESULT_1_0_DATA0_MASK);
	u16ADC1_SEG1 = (uint16_t)((ADC_ETC->TRIG[0].TRIGn_RESULT_1_0 & ADC_ETC_TRIGn_RESULT_1_0_DATA1_MASK)>>ADC_ETC_TRIGn_RESULT_1_0_DATA1_SHIFT);
	u16ADC2_SEG0 = (uint16_t)(ADC_ETC->TRIG[4].TRIGn_RESULT_1_0 & ADC_ETC_TRIGn_RESULT_1_0_DATA0_MASK);
	u16ADC2_SEG1 = (uint16_t)((ADC_ETC->TRIG[4].TRIGn_RESULT_1_0 & ADC_ETC_TRIGn_RESULT_1_0_DATA1_MASK)>>ADC_ETC_TRIGn_RESULT_1_0_DATA1_SHIFT);

    gsMC_Drive.sFocPMSM.sAllChnlOffsets.sIRaw.f16A = MLIB_Mul_F16(u16ADC1_SEG0<<3, FRAC16(0.58182))<<1;
    gsMC_Drive.sFocPMSM.sAllChnlOffsets.sIRaw.f16B = MLIB_Mul_F16(u16ADC2_SEG0<<3, FRAC16(0.58182))<<1;
	gsMC_Drive.sFocPMSM.f16UDcBus = MLIB_Mul_F16(u16ADC1_SEG1<<3, FRAC16(0.58182))<<1;
	gsMC_Drive.f16IPMTemp         = MLIB_Mul_F16(u16ADC2_SEG1<<3, FRAC16(0.58182))<<1; // Not available here

	/*========================= Read currents/DC bus voltage/IPM temperature end  =========================*/

    /*========================= State machine begin========================================================*/

	/* Slow loop calculation */
	geMC_StateRunLoop = SLOW;

	/*StateMachine call */
	SM_StateMachine(&gsMC_Ctrl);

	/* Fast loop calculation */
	geMC_StateRunLoop = FAST;

	/* StateMachine call */
	SM_StateMachine(&gsMC_Ctrl);

    /*use ADC channel mapping After calibration*/
	/* Channel update */
	/* S0  S1  S2  S3   S4  S5  S6  S7 */
	/* IX  Udc
	 *                    */
	ADC_ETC->TRIG[0].TRIGn_CHAIN_1_0 &= ~ADC_ETC_TRIGn_CHAIN_1_0_CSEL0_MASK;
	ADC_ETC->TRIG[4].TRIGn_CHAIN_1_0 &= ~ADC_ETC_TRIGn_CHAIN_1_0_CSEL0_MASK;

	ADC_ETC->TRIG[0].TRIGn_CHAIN_1_0 |= ADC_ETC_TRIGn_CHAIN_1_0_CSEL0(gsMC_Drive.sFocPMSM.sAllChnlOffsets.sIChnl.u16ADC1_CMD);
	ADC_ETC->TRIG[4].TRIGn_CHAIN_1_0 |= ADC_ETC_TRIGn_CHAIN_1_0_CSEL0(gsMC_Drive.sFocPMSM.sAllChnlOffsets.sIChnl.u16ADC2_CMD);

   	/*========================= State machine end =========================================================*/

	w16ExeTime = TMR1->CHANNEL[0].CNTR;

#if FMSTR_ON == 1
   /* Recorder of FreeMASTER*/
	FMSTR_Recorder(0);
#endif

//	GPIO_PortToggle(BOARD_INITPINS_LED_RED_GPIO, BOARD_INITPINS_LED_RED_GPIO_PIN_MASK);
    if(++u32Cnt == 8000)
    {
    	u32Cnt = 0;
    	GPIO_PortToggle(BOARD_INITPINS_LED_RED_GPIO, BOARD_INITPINS_LED_RED_GPIO_PIN_MASK);
    }


    __DSB();
}


