/*
 * Copyright 2016-2021 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file    MIMXRT1176_Project_M7.c
 * @brief   Application entry point.
 */
#include "peripherals_manual.h"
#include <stdio.h>
#include "board.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "MIMXRT1176_cm7.h"
#include "fsl_debug_console.h"
#include "button.h"
#include "MC_statemachine.h"
#include "hwcontrol.h"
/* TODO: insert other include files here. */

/* TODO: insert other definitions and declarations here. */
//-------------------- Global variables ------------------------
BUTTON_CTRL_TYPE sSW; // button SW

//----------------------- Variable declaration ----------------------
extern MCSTRUC_FAULT_RECORD_T	faultRecordFlag;		// Fault recorder variable
extern uint16_t								uw16LedCntBlkTimes;
extern uint16_t								uw16LedCntBlkDuration;
extern uint16_t								uw16LedCntPatternTimes;
extern uint16_t								uw16LedCntPatternSpace;

void pit1Init(void)
{
	CLOCK_EnableClock(kCLOCK_Pit1);
	PIT1->MCR &= ~PIT_MCR_MDIS_MASK;
	PIT1->CHANNEL[0].LDVAL = CLOCK_GetFreqFromObs(CCM_OBS_BUS_CLK_ROOT)/10000; // 10kHz
	PIT1->CHANNEL[0].TFLG = 1; // Clear flag
	PIT1->CHANNEL[0].TCTRL = PIT_TCTRL_TIE_MASK; // Enable interrupt of channel0 timer

//	NVIC_SetPriority(PIT1_IRQn, 0); // Priority 0
//	NVIC_EnableIRQ(PIT1_IRQn);
}


/*
 * @brief   Application entry point.
 */
int main(void) {

	BOARD_ConfigMPU();
    /* Init board hardware. */
    BOARD_InitBootPins();
    BOARD_InitBootClocks();
    BOARD_InitBootPeripherals();

    peripherals_manual_init();
    pit1Init();
    PIT1->CHANNEL[0].TCTRL |= PIT_TCTRL_TEN_MASK;

    /* Initialize buttons */
    sSW.sStatus = NOT_PRESSED;
    sSW.readButtonVal = readSWButton;
    sSW.check_button = is_button_pressed;
    sSW.debounce = button_process;


    /* Set speed command to zero */
//    gsMC_Drive.sSpeed.f16SpeedCmd = 0;

    /* Enable the counter of PWMA to start the state machine */
    PWMA_SM012_RUN();

    /* Led indication variables */
    uw16LedCntPatternSpace = 0;
    uw16LedCntBlkDuration = 0;
    uw16LedCntPatternTimes = 1;
    uw16LedCntBlkTimes = 1;

    gsMC_Drive.sSpeed.f16SpeedCmd = FRAC16(2000.0/N_MAX);
    /* Enter an infinite loop, just incrementing a counter. */
    while(1)
    {
    	  FMSTR_Poll();
		  gsMC_Drive.sSpeed.f16SpeedReq = gsMC_Drive.sSpeed.f16SpeedCmd;
    }
    return 0 ;
}





