/*
* Copyright 2019 NXP
* SPDX-License-Identifier: BSD-3-Clause
*/

#include "button.h"
#include "fsl_gpio.h"
#include "pin_mux.h"

uint16_t readSWButton(BUTTON_CTRL_TYPE *ptr)
{
	return (GPIO_PinRead(BOARD_INITPINS_USER_BUTTON_GPIO, BOARD_INITPINS_USER_BUTTON_GPIO_PIN));
}


uint16_t is_button_pressed(BUTTON_CTRL_TYPE *ptr)
{
	uint16_t u16Val = 0;

	switch(ptr->sStatus)
	{
	case NOT_PRESSED:
		if( ptr->readButtonVal(ptr) == 0)
		{
			ptr->sStatus = PRESSED_BEFORE_DEBOUNCE;
		}
		break;
	case PRESSED_BEFORE_DEBOUNCE:
		if(ptr->u16Cnt > DEBOUNCE_CNT)
		{
			ptr->u16Cnt = 0;
			ptr->sStatus = PRESSED_AFTER_DEBOUNCE;
		}
		break;
	case PRESSED_AFTER_DEBOUNCE:
		if(ptr->readButtonVal(ptr) == 0) // still pressed
		{
			ptr->sStatus = WAIT_FOR_RELEASE;
		}
		else
		{
			ptr->sStatus = NOT_PRESSED;
		}			
		break;
	case WAIT_FOR_RELEASE:
		if(ptr->readButtonVal(ptr) != 0) // released
		{
			ptr->sStatus = RELEASED_BEFORE_DEBOUNCE;
		}
		break;
	case RELEASED_BEFORE_DEBOUNCE:
		if(ptr->u16Cnt > DEBOUNCE_CNT)
		{
			ptr->u16Cnt = 0;
			ptr->sStatus = RELEASED_AFTER_DEBOUNCE;
		}
		break;
	case RELEASED_AFTER_DEBOUNCE:
		if(ptr->readButtonVal(ptr) != 0) // released
		{
			u16Val = 1;
		}
		ptr->sStatus = NOT_PRESSED;
		break;
	default:
		break;
	}
	return u16Val;
}

void button_process(BUTTON_CTRL_TYPE *ptr)
{
	if((ptr->sStatus == PRESSED_BEFORE_DEBOUNCE)||(ptr->sStatus == RELEASED_BEFORE_DEBOUNCE))
	{
		ptr->u16Cnt++;
	}
}
