/*
* Copyright 2019 NXP
* SPDX-License-Identifier: BSD-3-Clause
*/

#include "peripherals_manual.h"
#include "fsl_device_registers.h"
#include "PMSM_SpeedVectorCtrl.h"
#include "fsl_common.h"
#include "fsl_clock.h"

ADC_CHANNEL_TABLE_T sADCChannal = {IA_ADC1, IB_ADC2, IC_ADC1, IC_ADC2, UDC_ADC1};

void eFlexPWM1_init(void)
{
	CLOCK_EnableClock(kCLOCK_Pwm1);

	PWM1->SM[0].CTRL |= PWM_CTRL_FULL_MASK; // full cycle reload
	PWM1->SM[0].CTRL &= ~PWM_CTRL_COMPMODE_MASK; // Edge is generated on counter "equal to" value register
	PWM1->SM[0].CTRL &= ~PWM_CTRL_LDMOD_MASK; // buffered registers take effect at PWM reload signal when LDOK is set
	PWM1->SM[0].CTRL2 = (PWM1->SM[0].CTRL2 & ~PWM_CTRL2_CLK_SEL_MASK)|PWM_CTRL2_CLK_SEL(0); // use IPBus clock
	PWM1->SM[0].CTRL2 &= ~PWM_CTRL2_INDEP_MASK; // complementary mode
	PWM1->SM[0].CTRL2 = (PWM1->SM[0].CTRL2 & ~PWM_CTRL2_INIT_SEL_MASK)|PWM_CTRL2_INIT_SEL(0); // use local sync for SM0
	PWM1->SM[0].CTRL2 &= ~PWM_CTRL2_RELOAD_SEL_MASK; // use local reload signal

	PWM1->SM[1].CTRL |= PWM_CTRL_FULL_MASK; // full cycle reload
	PWM1->SM[1].CTRL &= ~PWM_CTRL_COMPMODE_MASK; // Edge is generated on counter "equal to" value register
	PWM1->SM[1].CTRL &= ~PWM_CTRL_LDMOD_MASK; // buffered registers take effect at PWM reload signal when LDOK is set
	PWM1->SM[1].CTRL2 = (PWM1->SM[1].CTRL2 & ~PWM_CTRL2_CLK_SEL_MASK)|PWM_CTRL2_CLK_SEL(2); // SM0's clock is used
	PWM1->SM[1].CTRL2 &= ~PWM_CTRL2_INDEP_MASK; // complementary mode
	PWM1->SM[1].CTRL2 = (PWM1->SM[1].CTRL2 & ~PWM_CTRL2_INIT_SEL_MASK)|PWM_CTRL2_INIT_SEL(2); // use master sync from SM0
	PWM1->SM[1].CTRL2 |= PWM_CTRL2_RELOAD_SEL_MASK; // use master reload from SM0

	PWM1->SM[2].CTRL |= PWM_CTRL_FULL_MASK; // full cycle reload
	PWM1->SM[2].CTRL &= ~PWM_CTRL_COMPMODE_MASK; // Edge is generated on counter "equal to" value register
	PWM1->SM[2].CTRL &= ~PWM_CTRL_LDMOD_MASK; // buffered registers take effect at PWM reload signal when LDOK is set
	PWM1->SM[2].CTRL2 = (PWM1->SM[2].CTRL2 & ~PWM_CTRL2_CLK_SEL_MASK)|PWM_CTRL2_CLK_SEL(2); // SM0's clock is used
	PWM1->SM[2].CTRL2 &= ~PWM_CTRL2_INDEP_MASK; // complementary mode
	PWM1->SM[2].CTRL2 = (PWM1->SM[2].CTRL2 & ~PWM_CTRL2_INIT_SEL_MASK)|PWM_CTRL2_INIT_SEL(2); // use master sync from SM0
	PWM1->SM[2].CTRL2 |= PWM_CTRL2_RELOAD_SEL_MASK; // use master reload from SM0


	PWM1->SM[0].INIT  = -((int16_t)(CLOCK_GetFreqFromObs(CCM_OBS_BUS_CLK_ROOT)/(2.0*MC_FAST_CONTROL_LOOP_FREQ))); // 16KHz @bus root clock
	PWM1->SM[0].VAL1  = CLOCK_GetFreqFromObs(CCM_OBS_BUS_CLK_ROOT)/(2.0*MC_FAST_CONTROL_LOOP_FREQ)-1;
	PWM1->SM[0].VAL0 = 0;
	PWM1->SM[0].VAL2  = PWM1->SM[0].INIT/2;
	PWM1->SM[0].VAL3  = PWM1->SM[0].VAL1/2;
	PWM1->SM[0].VAL4 = PWM1->SM[0].INIT + ADC_TRIGGER_DELAY*CLOCK_GetFreqFromObs(CCM_OBS_BUS_CLK_ROOT)/1000000;

	PWM1->SM[1].INIT  = PWM1->SM[0].INIT; // 16KHz @bus root clock
	PWM1->SM[1].VAL1  = PWM1->SM[0].VAL1;
	PWM1->SM[1].VAL0 = 0;
	PWM1->SM[1].VAL2  = PWM1->SM[0].VAL2;
	PWM1->SM[1].VAL3  = PWM1->SM[0].VAL3;

	PWM1->SM[2].INIT  = PWM1->SM[0].INIT; // 16KHz @bus root clock
	PWM1->SM[2].VAL1  = PWM1->SM[0].VAL1;
	PWM1->SM[2].VAL0 = 0;
	PWM1->SM[2].VAL2  = PWM1->SM[0].VAL2;
	PWM1->SM[2].VAL3  = PWM1->SM[0].VAL3;

	PWM1->SM[0].DTCNT0 = PWM_DEADTIME*CLOCK_GetFreqFromObs(CCM_OBS_BUS_CLK_ROOT)/1000000;
	PWM1->SM[0].DTCNT1 = PWM_DEADTIME*CLOCK_GetFreqFromObs(CCM_OBS_BUS_CLK_ROOT)/1000000;
	PWM1->SM[1].DTCNT0 = PWM_DEADTIME*CLOCK_GetFreqFromObs(CCM_OBS_BUS_CLK_ROOT)/1000000;
	PWM1->SM[1].DTCNT1 = PWM_DEADTIME*CLOCK_GetFreqFromObs(CCM_OBS_BUS_CLK_ROOT)/1000000;
	PWM1->SM[2].DTCNT0 = PWM_DEADTIME*CLOCK_GetFreqFromObs(CCM_OBS_BUS_CLK_ROOT)/1000000;
	PWM1->SM[2].DTCNT1 = PWM_DEADTIME*CLOCK_GetFreqFromObs(CCM_OBS_BUS_CLK_ROOT)/1000000;

	PWM1->SM[0].DISMAP[0] = 0;
	PWM1->SM[1].DISMAP[0] = 0;
	PWM1->SM[2].DISMAP[0] = 0;

	PWM1->SM[0].TCTRL = (PWM1->SM[0].TCTRL & ~PWM_TCTRL_OUT_TRIG_EN_MASK)|PWM_TCTRL_OUT_TRIG_EN(0x10); // Use SM0_Val4 for trigger point

	/* Fault0 protects PWM1&PWMB outputs of SM0~SM2 */
//	PWM1->SM[0].DISMAP[0] = (PWM1->SM[0].DISMAP[0] & ~(PWM_DISMAP_DIS0A_MASK|PWM_DISMAP_DIS0B_MASK))|PWM_DISMAP_DIS0A(1)|PWM_DISMAP_DIS0B(1);
//	PWM1->SM[1].DISMAP[0] = (PWM1->SM[0].DISMAP[0] & ~(PWM_DISMAP_DIS0A_MASK|PWM_DISMAP_DIS0B_MASK))|PWM_DISMAP_DIS0A(1)|PWM_DISMAP_DIS0B(1);
//	PWM1->SM[2].DISMAP[0] = (PWM1->SM[0].DISMAP[0] & ~(PWM_DISMAP_DIS0A_MASK|PWM_DISMAP_DIS0B_MASK))|PWM_DISMAP_DIS0A(1)|PWM_DISMAP_DIS0B(1);

	PWM1->FCTRL = (PWM1->FCTRL & ~PWM_FCTRL_FLVL_MASK)|PWM_FCTRL_FLVL(1); // A logic 1 on fault0 indicates a fault condition
	PWM1->FCTRL = (PWM1->FCTRL & ~PWM_FCTRL_FAUTO_MASK)|PWM_FCTRL_FAUTO(0); // Manual clearing for fault0
	PWM1->FCTRL = (PWM1->FCTRL & ~PWM_FCTRL_FSAFE_MASK)|PWM_FCTRL_FSAFE(1); // Safe mode for fault0

	PWM1->FSTS = (PWM1->FSTS & ~PWM_FSTS_FFULL_MASK)|PWM_FSTS_FFULL(1); // Full cycle recovery for fault0
	PWM1->FCTRL2 = (PWM1->FCTRL2 & ~PWM_FCTRL2_NOCOMB_MASK)|PWM_FCTRL2_NOCOMB(1); // No combinational path for fault0. Fault0 signal will be filtered
	PWM1->FFILT = PWM_FFILT_FILT_CNT(3)|PWM_FFILT_FILT_PER(2);

	PWM1->MCTRL = PWM_MCTRL_CLDOK(0x7); // Clear LDOK of SM0~SM2
	PWM1->MCTRL |= PWM_MCTRL_LDOK(0x7); // Set LDOK of SM0~SM2

//	PWM1->SM[0].INTEN = PWM_INTEN_RIE_MASK; // SM0 Reload interrupt enable
//	NVIC_SetPriority(PWM1_0_IRQn, 1); // Priority 1
//	NVIC_EnableIRQ(PWM1_0_IRQn);

}



void lpadc_init(void)
{
	CLOCK_EnableClock(kCLOCK_Lpadc1);
	CLOCK_EnableClock(kCLOCK_Lpadc2);

	LPADC1->CTRL = ADC_CTRL_RSTFIFO_MASK|ADC_CTRL_RST_MASK; // Reset fifo and wrapper logic
	LPADC1->CTRL = ADC_CTRL_TRIG_SRC(0); // ADC_ETC trigger only
	LPADC2->CTRL = ADC_CTRL_RSTFIFO_MASK|ADC_CTRL_RST_MASK; // Reset fifo and wrapper logic
	LPADC2->CTRL = ADC_CTRL_TRIG_SRC(0); // ADC_ETC trigger only

	LPADC1->CFG |= ADC_CFG_PWREN(1)|ADC_CFG_REFSEL(0)|ADC_CFG_PWRSEL(3); // External VREFH=1.8V, highest power consumption
	LPADC2->CFG |= ADC_CFG_PWREN(1)|ADC_CFG_REFSEL(0)|ADC_CFG_PWRSEL(3); // External VREFH=1.8V, highest power consumption

	LPADC1->CMD[0].CMDL &= ~ADC_CMDL_CSCALE_MASK; // Input is scaled to 30/64
	LPADC1->CMD[1].CMDL &= ~ADC_CMDL_CSCALE_MASK; // Input is scaled to 30/64
	LPADC1->CMD[2].CMDL &= ~ADC_CMDL_CSCALE_MASK; // Input is scaled to 30/64

	LPADC2->CMD[0].CMDL &= ~ADC_CMDL_CSCALE_MASK; // Input is scaled to 30/64
	LPADC2->CMD[1].CMDL &= ~ADC_CMDL_CSCALE_MASK; // Input is scaled to 30/64
	LPADC2->CMD[2].CMDL &= ~ADC_CMDL_CSCALE_MASK; // Input is scaled to 30/64

	LPADC1->CMD[0].CMDL |= ADC_CMDL_ABSEL(sADCChannal.sIA_ADC1.u16Side)|ADC_CMDL_ADCH(sADCChannal.sIA_ADC1.u16Num);   // ADC1_CMD1 is linked to Ia_adc1
	LPADC1->CMD[1].CMDL |= ADC_CMDL_ABSEL(sADCChannal.sIC_ADC1.u16Side)|ADC_CMDL_ADCH(sADCChannal.sIC_ADC1.u16Num);   // ADC1_CMD2 is linked to Ic_adc1
	LPADC1->CMD[2].CMDL |= ADC_CMDL_ABSEL(sADCChannal.sUDC_ADC1.u16Side)|ADC_CMDL_ADCH(sADCChannal.sUDC_ADC1.u16Num); // ADC1_CMD3 is linked to Udc_adc1


	LPADC2->CMD[0].CMDL |= ADC_CMDL_ABSEL(sADCChannal.sIB_ADC2.u16Side)|ADC_CMDL_ADCH(sADCChannal.sIB_ADC2.u16Num);   // ADC2_CMD1 is linked to Ib_adc2
	LPADC2->CMD[1].CMDL |= ADC_CMDL_ABSEL(sADCChannal.sIC_ADC2.u16Side)|ADC_CMDL_ADCH(sADCChannal.sIC_ADC2.u16Num);   // ADC2_CMD2 is linked to Ic_adc2



	LPADC1->TCTRL[0] = ADC_TCTRL_CMD_SEL(1)|ADC_TCTRL_HTEN_MASK; // Enable HW trigger for TRIG0 of ADC1, TCMD comes from ADC_ETC
	LPADC2->TCTRL[0] = ADC_TCTRL_CMD_SEL(1)|ADC_TCTRL_HTEN_MASK; // Enable HW trigger for TRIG0 of ADC2, TCMD comes from ADC_ETC
//	LPADC1->TCTRL[1] = ADC_TCTRL_CMD_SEL(1)|ADC_TCTRL_HTEN_MASK; // Enable HW trigger for TRIG1 of ADC1, TCMD comes from ADC_ETC
//	LPADC2->TCTRL[1] = ADC_TCTRL_CMD_SEL(1)|ADC_TCTRL_HTEN_MASK; // Enable HW trigger for TRIG1 of ADC2, TCMD comes from ADC_ETC

	LPADC1->CTRL |= ADC_CTRL_ADCEN_MASK;
	LPADC2->CTRL |= ADC_CTRL_ADCEN_MASK;
}


void adc_etc_init(void)
{
	CLOCK_EnableClock(kCLOCK_Adc_Etc);

	ADC_ETC->CTRL = ADC_ETC_CTRL_SOFTRST_MASK; // Reset adc_etc
	ADC_ETC->CTRL &= ~ADC_ETC_CTRL_SOFTRST_MASK;

	ADC_ETC->CTRL = ADC_ETC_CTRL_TRIG_ENABLE(0x1); // TRIG0 is enabled

	ADC_ETC->TRIG[0].TRIGn_CTRL = 0;
	ADC_ETC->TRIG[0].TRIGn_CTRL = ADC_ETC_TRIGn_CTRL_TRIG_CHAIN(1)|\
								  ADC_ETC_TRIGn_CTRL_SYNC_MODE_MASK|\
								  ADC_ETC_TRIGn_CTRL_TRIG_MODE(0); // HW trigger,sync mode, chain length is 2
	ADC_ETC->TRIG[4].TRIGn_CTRL = 0;
	ADC_ETC->TRIG[4].TRIGn_CTRL = ADC_ETC_TRIGn_CTRL_TRIG_CHAIN(1)|\
								  ADC_ETC_TRIGn_CTRL_SYNC_MODE_MASK|\
								  ADC_ETC_TRIGn_CTRL_TRIG_MODE(0); // HW trigger,sync mode, chain length is 2

	/*
	 * 				                 	SEG0     SEG1
	 * 				LPADC1 on TRIG0 : IA_ADC1, UDC_ADC1
	 *
	 * 				LPADC2 on TRIG4 : IB_ADC2, IC_ADC2
	 *
	 *
	 * */
	ADC_ETC->TRIG[0].TRIGn_CHAIN_1_0 = ADC_ETC_TRIGn_CHAIN_1_0_B2B1_MASK|ADC_ETC_TRIGn_CHAIN_1_0_HWTS1(1)|ADC_ETC_TRIGn_CHAIN_1_0_CSEL1(sADCChannal.sUDC_ADC1.u16CMD)|\
										ADC_ETC_TRIGn_CHAIN_1_0_B2B0_MASK|ADC_ETC_TRIGn_CHAIN_1_0_HWTS0(1)|ADC_ETC_TRIGn_CHAIN_1_0_CSEL0(sADCChannal.sIA_ADC1.u16CMD);
	ADC_ETC->TRIG[4].TRIGn_CHAIN_1_0 = ADC_ETC_TRIGn_CHAIN_1_0_B2B1_MASK|ADC_ETC_TRIGn_CHAIN_1_0_HWTS1(1)|ADC_ETC_TRIGn_CHAIN_1_0_CSEL1(sADCChannal.sIC_ADC2.u16CMD)|\
										ADC_ETC_TRIGn_CHAIN_1_0_HWTS0(1)|ADC_ETC_TRIGn_CHAIN_1_0_CSEL0(sADCChannal.sIB_ADC2.u16CMD)|\
										ADC_ETC_TRIGn_CHAIN_1_0_B2B0_MASK|ADC_ETC_TRIGn_CHAIN_1_0_IE1(2)|ADC_ETC_TRIGn_CHAIN_1_0_IE1_EN_MASK; // Generates an interrupt on DONE2 of TRIG4 when segment 1 is finished


	NVIC_SetPriority(ADC_ETC_IRQ2_IRQn, 0); // Priority 0
	NVIC_EnableIRQ(ADC_ETC_IRQ2_IRQn);

}

void qtimer1_0_init(void)
{
	CLOCK_EnableClock(kCLOCK_Qtimer1);

	TMR1->CHANNEL[0].CTRL = TMR_CTRL_PCS(8)|TMR_CTRL_CM(1); // IP bus clock
}

void peripherals_manual_init(void)
{
	/* Must enable clock gates of used peripherals before configuring them */
	eFlexPWM1_init();
	lpadc_init();
	adc_etc_init();
	qtimer1_0_init();
}



