/*
* Copyright 2019 NXP
* SPDX-License-Identifier: BSD-3-Clause
*/

#ifndef PERIPHERALS_MANUAL_H_
#define PERIPHERALS_MANUAL_H_

#include "fsl_gpio.h"
#include "mlib.h"
#include "pin_mux.h"

extern void peripherals_manual_init(void);

#define MASK_BIT15 0x8000
#define MASK_BIT14 0x4000
#define MASK_BIT13 0x2000
#define MASK_BIT12 0x1000
#define MASK_BIT11 0x0800
#define MASK_BIT10 0x0400
#define MASK_BIT9 0x0200
#define MASK_BIT8 0x0100
#define MASK_BIT7 0x0080
#define MASK_BIT6 0x0040
#define MASK_BIT5 0x0020
#define MASK_BIT4 0x0010
#define MASK_BIT3 0x0008
#define MASK_BIT2 0x0004
#define MASK_BIT1 0x0002
#define MASK_BIT0 0x0001

#define LED_RED_OFF() 		GPIO_PortClear(BOARD_INITPINS_LED_RED_GPIO, BOARD_INITPINS_LED_RED_GPIO_PIN_MASK)
#define LED_RED_ON()  		GPIO_PortSet(BOARD_INITPINS_LED_RED_GPIO, BOARD_INITPINS_LED_RED_GPIO_PIN_MASK)
#define LED_RED_TOGGLE() 	GPIO_PortToggle(BOARD_INITPINS_LED_RED_GPIO, BOARD_INITPINS_LED_RED_GPIO_PIN_MASK)
#define LED_RED_PUTVAL(val) GPIO_PinWrite(BOARD_INITPINS_LED_RED_GPIO, BOARD_INITPINS_LED_RED_GPIO_PIN, val)

#define PWM_DEADTIME		1.5  // us
#define ADC_TRIGGER_DELAY 	1.25 // us

#define A_SIDE	0
#define B_SIDE  1

#define IA_ADC1 {2,A_SIDE,1}
#define IB_ADC2 {3,A_SIDE,1}
#define IC_ADC1 {3,B_SIDE,2}
#define IC_ADC2 {3,B_SIDE,2}
#define UDC_ADC1 {2,B_SIDE,3}

typedef struct
{
	uint16_t u16Num;
	uint16_t u16Side;
	uint16_t u16CMD;
}ADC_CHANNEL_NUM_T;

typedef struct
{
	ADC_CHANNEL_NUM_T sIA_ADC1;
	ADC_CHANNEL_NUM_T sIB_ADC2;
	ADC_CHANNEL_NUM_T sIC_ADC1;
	ADC_CHANNEL_NUM_T sIC_ADC2;
	ADC_CHANNEL_NUM_T sUDC_ADC1;
}ADC_CHANNEL_TABLE_T;

#endif /* PERIPHERALS_MANUAL_H_ */
